use clap::{App, Arg, ArgMatches};
use nix::sys::inotify::{AddWatchFlags, InitFlags, Inotify};
use serde::{Deserialize, Serialize};
use serde_json::Error;
use std::string::String;
use std::thread::sleep;
use std::time::Duration;
use std::{fs::File, io::Read};

#[derive(Debug, Clone)]
pub struct Context {
    pub cfg: Config,
    pub configfile: String,
    pub inotify: Inotify,
}

impl Context {
    pub fn new() -> Self {
        let configfile = Context::argparse()
            .value_of("config")
            .unwrap_or("config.json")
            .to_string();

        let mut ctx = Context {
            cfg: Config::new(),
            configfile: configfile,
            inotify: Inotify::init(InitFlags::IN_NONBLOCK).unwrap(),
        };

        println!("Loading {} file ...", ctx.configfile);
        ctx.cfg.load(&ctx.configfile);

        println!("Adding inotify watch on {} file ...", ctx.configfile);
        ctx.inotify
            .add_watch(ctx.configfile.as_str(), AddWatchFlags::IN_MODIFY)
            .unwrap();

        ctx
    }
    pub fn hotreload(&mut self) {
        let events = match self.inotify.read_events() {
            Ok(ev) => ev,
            Err(_) => vec![],
        };
        if events.len() > 0 {
            self.cfg.load(self.configfile.as_str());
            println!("{:?} {} {:?}", self.cfg, events.len(), events);
        } else {
            sleep(Duration::from_secs(self.cfg.refresh.unwrap_or(5)));
        }
    }
    pub fn argparse() -> ArgMatches<'static> {
        App::new("Zabbix Launch")
            .version("1.0")
            .author("PaulBSD <paul@paulbsd.com>")
            .about("Lights up Launchpad mini using Zabbix data")
            .arg(
                Arg::with_name("config")
                    .short("c")
                    .long("config")
                    .value_name("FILE")
                    .help("Sets a custom config file")
                    .takes_value(true),
            )
            .get_matches()
    }
}

#[derive(Debug, Deserialize, Serialize, Clone)]
pub struct Config {
    pub server: String,
    pub username: String,
    pub password: String,
    pub authtoken: Option<String>,
    pub sloweffect: Option<bool>,
    pub refresh: Option<u64>,
    pub limit: Option<u64>,
}

impl<'a> Config {
    pub fn new() -> Self {
        Self {
            server: String::from("https://zabbix.acme.com/api_jsonrpc.php"),
            username: String::from("bob"),
            password: String::from("password"),
            authtoken: Some(String::from("token")),
            sloweffect: Some(true),
            refresh: Some(5u64),
            limit: Some(20u64),
        }
    }

    fn feed_missing(&mut self) -> bool {
        let mut res = false;
        if self.authtoken.is_none() {
            self.authtoken = Some(String::from(""));
            res = true;
        }
        if self.sloweffect.is_none() {
            self.sloweffect = Some(true);
            res = true;
        }
        if self.refresh.is_none() {
            self.refresh = Some(5u64);
            res = true;
        }
        if self.limit.is_none() {
            self.limit = Some(20u64);
            res = true;
        }
        res
    }

    pub fn load(&'a mut self, configfile: &str) {
        let fileopen: Result<File, std::io::Error>;
        let filemeta = std::fs::metadata(configfile);
        let fileexists = match filemeta {
            Ok(_) => true,
            Err(_err) => false,
        };

        if !fileexists {
            let _a = File::create(configfile);
        }
        fileopen = File::open(configfile);

        let mut file = match fileopen {
            Ok(f) => f,
            Err(e) => {
                panic!("{}", e);
            }
        };
        let mut contents = String::new();
        file.read_to_string(&mut contents).unwrap();
        let parse: Result<Config, Error> = serde_json::from_str(contents.as_str());
        let cfg = match parse {
            Ok(cfg) => cfg,
            Err(_e) => Config::new(),
        };
        *self = cfg;
        if self.feed_missing() {
            self.save(&configfile);
        };
    }

    pub fn save(&self, configfile: &str) {
        let file = File::create(configfile).unwrap();
        serde_json::to_writer_pretty(file, &self).unwrap();
    }
}
